#include "cpu_mon.hpp"

#include <chrono>
#include <fstream>
#include <iostream>
#include <sstream>
#include <thread>

CpuMon::CpuMon(uint8_t num_hilos, WINDOW *ventanaMonitorCPU) {
  CpuMon::num_hilos = num_hilos;
  CpuMon::ventanaMonitorCPU = ventanaMonitorCPU;
  CpuMon::enFuncionamiento = true;

  uint8_t num_sectores = 1;
  int lineas = ui::g_maxLines;

  while (lineas < (CpuMon::num_hilos / num_sectores)) {
    num_sectores++;
  }

  CpuMon::num_sectores = num_sectores;
  CpuMon::columnas = (ui::g_maxCols / 2) / CpuMon::num_sectores;
}

void CpuMon::esperar(int segundos) {
  std::this_thread::sleep_for(std::chrono::seconds(segundos));
}

void CpuMon::setNum_sectores(uint8_t num_sectores) {
  CpuMon::num_sectores = num_sectores;
}

void CpuMon::iniciarCPU_MON() {
  std::vector<DatosCPU> entries1;
  std::vector<DatosCPU> entries2;
  while (CpuMon::enFuncionamiento) {
    std::thread dormir(CpuMon::esperar, 1);

    // snapshot 1
    leerEstadoCPU(entries1);

    // 100ms pause
    std::this_thread::sleep_for(std::chrono::milliseconds(100));

    // snapshot 2
    leerEstadoCPU(entries2);

    // print output
    obtenerUsoCPU(entries1, entries2);

    dormir.join();
  }

  ui::borrarVentana(CpuMon::ventanaMonitorCPU);
}

void CpuMon::pararCPU_MON() { CpuMon::enFuncionamiento = false; }

// int CpuMon::getColumnas() { return CpuMon::columnas; }

// uint8_t CpuMon::getNum_sectores() { return CpuMon::num_sectores; }

void CpuMon::setEnfuncionamiento(bool enFuncionamiento) {
  CpuMon::enFuncionamiento = enFuncionamiento;
}

void CpuMon::leerEstadoCPU(std::vector<DatosCPU> &entradas) {
  std::ifstream fileStat("/proc/stat");

  std::string linea;

  const std::string STR_CPU("cpu");
  const std::size_t LEN_STR_CPU = STR_CPU.size();
  const std::string STR_TOT("tot");

  while (std::getline(fileStat, linea)) {
    // cpu stats line found
    if (!linea.compare(0, LEN_STR_CPU, STR_CPU)) {
      std::istringstream ss(linea);

      // store entry
      entradas.emplace_back(DatosCPU());
      DatosCPU &entry = entradas.back();

      // read cpu label
      ss >> entry.cpu;

      // remove "cpu" from the label when it's a processor number
      if (entry.cpu.size() > LEN_STR_CPU)
        entry.cpu.erase(0, LEN_STR_CPU);
      // replace "cpu" with "tot" when it's total values
      else
        entry.cpu = STR_TOT;

      // read times
      for (int i = 0; i < NUM_ESTADOS_CPU; ++i)
        ss >> entry.times[i];
    }
  }
}

size_t CpuMon::getTiempoInactivo(const DatosCPU &e) {
  return e.times[S_IDLE] + e.times[S_IOWAIT];
}

size_t CpuMon::getTiempoActivo(const DatosCPU &e) {
  return e.times[S_USER] + e.times[S_NICE] + e.times[S_SYSTEM] +
         e.times[S_IRQ] + e.times[S_SOFTIRQ] + e.times[S_STEAL] +
         e.times[S_GUEST] + e.times[S_GUEST_NICE];
}

void CpuMon::obtenerUsoCPU(const std::vector<DatosCPU> &entradas1,
                           const std::vector<DatosCPU> &entradas2) {

  const size_t NUM_ENTRADAS = entradas1.size();

  std::vector<std::string> salida(CpuMon::num_hilos);

  // Comienza en 1 para saltarse el total
  for (uint8_t i = 1; i < NUM_ENTRADAS; ++i) {
    const DatosCPU &e1 = entradas1[i];
    const DatosCPU &e2 = entradas2[i];

    std::cout.width(3);
    std::cout << e1.cpu << "] ";

    const float TIEMPO_ACTIVO =
        static_cast<float>(getTiempoActivo(e2) - getTiempoActivo(e1));
    const float TIEMPO_INACTIVO =
        static_cast<float>(getTiempoInactivo(e2) - getTiempoInactivo(e1));
    const float TIEMPO_TOTAL = TIEMPO_ACTIVO + TIEMPO_INACTIVO;

    float usoCPU = 1.f * TIEMPO_ACTIVO / TIEMPO_TOTAL;
    salida[i - 1] = CpuMon::obtenerBarra(&usoCPU, &i);
  }

  ui::mostrarVentana(ventanaMonitorCPU, &salida, &columnas, &num_sectores);
}

std::string CpuMon::obtenerBarra(float *porcentaje, uint8_t *cpu) {
  std::string barra = " CPU ";

  if (CpuMon::num_hilos > 9 && *cpu < 10) {
    barra.append("0").append(std::to_string(*cpu).append(" ["));
  } else {
    barra.append(std::to_string(*cpu).append(" ["));
  }
  std::string cierre = "] ";

  int ent_por = (int)(100 * *porcentaje);

  if (ent_por < 10) {
    cierre.append("  ");
  } else if (ent_por < 100) {
    cierre.append(" ");
  }

  cierre.append(std::to_string(ent_por));
  cierre.append(" %% ");

  // El ancho se calcula con el ancho máximo menos la longitud de los caracteres
  // iniciales y menos los carateres de cierre.
  int ancho = CpuMon::columnas - barra.length() - cierre.length() - 3;

  // Bucle para almohadillas
  int num_veces = (int)(ancho * *porcentaje);
  for (uint16_t i = 0; i < (int)(ancho * *porcentaje); i++) {
    barra.append("#");
  }

  // Bucle para guiones
  num_veces = ancho - num_veces;
  for (uint16_t i = 0; i < num_veces; i++) {
    barra.append("-");
  }

  barra.append(cierre);
  return barra;
}
